﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Windows.Input;

namespace WpfApplication1
{
    public abstract class CloosableViewModel : ViewModelBase, INotifyPropertyChanged
    {
        RelayCommand _closeCommand;
        RelayCommand _initCommand;
        RelayCommand _beforeCloseCommand;
        protected bool mModificado = false;

        //Para saber cuál es la VentanaActual del MainWindow (cual es el Pane "activo")
        static protected CloosableViewModel _viewModelActual = null;
        static public CloosableViewModel ViewModelActual
        {
            get { return _viewModelActual; }
            set
            {
                _viewModelActual = value;
            }
        }

        protected CloosableViewModel()
        {
            CancelClosingOperation = false;
        }

        public bool CancelClosingOperation
        {
            get;
            set;
        }

        #region PropiedadesModificado
        public virtual bool Modificado
        {
            get { return mModificado; }
        }
        #endregion

        #region Métodos privados
        public void MarcarNoModificado()
        {
            mModificado = false;
            OnPropertyChanged("DisplayName");
            OnPropertyChanged("Modificado");
        }

        protected void MarcarModificado()
        {
            mModificado = true;
            OnPropertyChanged("DisplayName");
            OnPropertyChanged("Modificado");
        }

        public override string DisplayName
        {
            get
            {
                if (mModificado)
                    return base.DisplayName + "*";
                else
                    return base.DisplayName;
            }
            set
            {
                base.DisplayName = value;
                OnPropertyChanged("DisplayName");
            }
        }
        #endregion

        #region CloseCommand
        public ICommand CloseCommand
        {
            get
            {
                if (_closeCommand == null)
                {
                    _closeCommand = new RelayCommand(
                           this.CerrarPestanya,
                           param => this._PuedeCerrarPestanya
                           );
                }

                return _closeCommand;
            }
        }


        private void CerrarPestanya(object pDatosPestanya)
        {
            //Mediator.NotifyColleagues("EliminarPestanya", pDatosPestanya);
            AntesCerrarPestanya(null);
            //Si se cierra desde la pestaña
            if (pDatosPestanya == this && CancelClosingOperation)
                return;

            OnRequestClose();
        }
        bool _PuedeCerrarPestanya = true;
        public bool PuedeCerrarPestanya
        {
            get { return _PuedeCerrarPestanya; }
            set { _PuedeCerrarPestanya = value; }
        }

        public event EventHandler RequestClose;

        internal void OnRequestClose()
        {
            EventHandler handler = this.RequestClose;
            if (handler != null)
                handler(this, EventArgs.Empty);
        }

        #endregion // CloseCommand

        #region InitCommand

        public ICommand InitCommand
        {
            get
            {
                if (_initCommand == null)
                {
                    _initCommand = new RelayCommand(
                           this.AbrirPestanya,
                           param => this.PuedeAbrirPestanya
                           );
                }

                return _initCommand;
            }
        }

        private void AbrirPestanya(object pDatosPestanya)
        {
            OnRequestInit();
        }
        public bool PuedeAbrirPestanya { get { return true; } }

        public event EventHandler RequestInit;

        internal void OnRequestInit()
        {
            EventHandler handler = this.RequestInit;
            if (handler != null)
                handler(this, EventArgs.Empty);
        }

        #endregion

        #region BeforeCloseCommand
        public ICommand BeforeCloseCommand
        {
            get
            {
                if (_beforeCloseCommand == null)
                {
                    _beforeCloseCommand = new RelayCommand(
                           this.AntesCerrarPestanya);
                }

                return _beforeCloseCommand;
            }
        }

        private void AntesCerrarPestanya(object pDatosPestanya)
        {
            OnRequestBeforeClose();
        }

        public event CancelEventHandler RequestBeforeClose;

        internal void OnRequestBeforeClose()
        {
            CancelEventHandler handler = this.RequestBeforeClose;
            if (handler != null)
            {
                CancelEventArgs eventArgs = new CancelEventArgs();
                handler(this, eventArgs);

                CancelClosingOperation = eventArgs.Cancel;
            }
        }
        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName, bool EsDatoPersistencia = true)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                var e = new PropertyChangedEventArgs(propertyName);
                handler(this, e);
            }
        }

        #endregion // INotifyPropertyChanged Members

    }
}