﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.ComponentModel;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Media;

namespace WpfApplication1
{
    public class RelayCommand : ICommand, INotifyPropertyChanged
    {
        readonly Action<object> _execute;
        readonly Action<object, object> _execute2;
        readonly Predicate<object> _canExecute;

        Image mIcono = new Image();

        #region constructores
        public RelayCommand(Action<object> execute, Uri pUriIcono = null)
            : this(execute, null, pUriIcono)
        {
        }

        public RelayCommand(Action<object> execute, Predicate<object> canExecute, Uri pUriIcono = null)
        {
            if (execute == null)
                throw new ArgumentNullException("execute");

            _execute = execute;
            _canExecute = canExecute;
            InicializarIcono(pUriIcono);
        }

        public RelayCommand(Action<object, object> execute2, Predicate<object> canExecute, Uri pUriIcono = null)
        {
            if (execute2 == null)
                throw new ArgumentNullException("execute");

            _execute2 = execute2;
            _canExecute = canExecute;
            InicializarIcono(pUriIcono);
        }
        #endregion
        #region Propiedades para la view

        public Image Icono
        {
            get { return mIcono; }
            set
            {
                mIcono = value;
                OnPropertyChanged("Icono");
            }
        }

        #endregion

        #region Metodos privados

        private void InicializarIcono(Uri pUriIcono)
        {
            if (pUriIcono != null)
            {
                Icono = new Image();
                BitmapImage bi1 = new BitmapImage();
                bi1.BeginInit();
                bi1.UriSource = pUriIcono;
                bi1.EndInit();
                bi1.Freeze();
                Icono.Stretch = Stretch.Fill;
                Icono.Source = bi1;
            }
        }

        #endregion

        #region ICommand Members

        public bool CanExecute(object parameter)
        {
            return _canExecute == null ? true : _canExecute(parameter);
        }

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public void Execute(object parameter, object parameter2)
        {
            if (_execute2 != null)
                _execute2(parameter, parameter2);
        }

        public void Execute(object parameter)
        {
            if (_execute != null && CanExecute(parameter))
                _execute(parameter);
            else if (_execute2 != null)
                _execute2(parameter, null);
        }

        #endregion // ICommand Members

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                var e = new PropertyChangedEventArgs(propertyName);
                handler(this, e);
            }
        }

        #endregion // INotifyPropertyChanged Members

    }
}
